@@ -3,6 +3,7 @@ import re
 import uuid
 
 import orjson
+import ujson
 
 from firepit import raft
 from firepit.exceptions import IncompatibleType
@@ -18,20 +19,26 @@ from firepit.validate import validate_path
 
 logger = logging.getLogger(__name__)
 
-STIX_TRANSFORMS = [raft.preserve, raft.invert, raft.markroot,
-                   raft.makeid, raft.nest, raft.promote, raft.normalize]
 
-
-def _transform(ops, filename):
+def _transform(filename):
     for obj in raft.get_objects(filename, ['identity', 'observed-data']):
-        inputs = [obj]
-        for op in ops:
-            results = []
-            for i in inputs:
-                results.extend(op(i))
-            inputs = results
-        for result in results:
-            yield result
+        if obj['type'] == 'observed-data':
+            obj['x_stix'] = ujson.dumps(obj)  # raft.preserve
+            obj = raft.invert(obj)
+            obj = raft.markroot(obj)  # Also does raft.makeid now
+            observables = obj.get('objects', {})
+            # Inlined below: obj = raft.nest(obj)
+            object_refs = []
+            for obs_orig in observables.values():
+                object_refs.append(raft._resolve(obs_orig, observables))
+            obj['objects'] = object_refs
+            objs = raft.promote(obj)
+            # Inlined below: raft.normalize
+            objs = [raft.json_normalize(obj, flat_lists=False) for obj in objs]
+            for obj in objs:
+                yield obj
+        else:
+            yield obj
 
 
 def infer_type(key, value):
@@ -251,7 +258,7 @@ class SqlStorage:
                     f' VALUES ({self.placeholder}, {self.placeholder})')
             cursor.execute(stmt, (obj['id'], query_id))
 
-    def upsert_many(self, cursor, tablename, objs, query_id):
+    def upsert_many(self, cursor, tablename, objs, query_id, schema):
         for obj in objs:
             self.upsert(cursor, tablename, obj, query_id)
 
@@ -269,7 +276,7 @@ class SqlStorage:
         for bundle in bundles:
             if isinstance(bundle, str):
                 logger.debug('- Caching %s', bundle)
-            for obj in _transform(STIX_TRANSFORMS, bundle):
+            for obj in _transform(bundle):
                 splitter.write(obj)
         splitter.close()
 
@@ -339,7 +346,7 @@ class SqlStorage:
             self._execute(f'DROP VIEW IF EXISTS "{viewname}"', cursor)
             columns = {k: self.infer_type(k, v) for k, v in objects[0].items()}
             self._create_table(viewname, columns)
-            self.upsert_many(cursor, viewname, objects, None)
+            self.upsert_many(cursor, viewname, objects, None, columns)
             viewdef = self._select(viewname)
         else:
             writer = self._get_writer(None)
